import 'dart:async';
import 'dart:html';
import 'dart:js' as js;
import 'dart:convert';
import 'package:nui_miniprogram_core/nui_miniprogram_engine.dart';
import 'package:nui_miniprogram_core/nui_miniprogram_enum.dart';
import 'package:nui_miniprogram_core/nui_miniprogram_message.dart';

class NUIMiniProgramWebJSEngine with NUIMiniProgramJSEngineInterface{
  NUIMiniProgramWebJSEngine._builder();
  Window _window;
  bool _connectionInitialized = false;

  bool get connectionInitialized{
    return _connectionInitialized;
  }

  static NUIMiniProgramWebJSEngine from(Window window){
    return NUIMiniProgramWebJSEngine._builder().._window = window;
  }

  void initCrossEventListener(NUICrossEventMessageListener listener){
    _window.addEventListener("message", (ev){
      MessageEvent messageEvent = ev;
      print("[NUIMiniProgramJSEngine] Message Event: ${messageEvent.data}, window name: ${window.name}");
      messageReceived(messageEvent.data);
    });
  }

  Future<bool> sendDefaultChannelEvent(NUIMiniProgramEventType type, {String payload}) async{
    final internalMessage = NUIMiniProgramInternalMessage(type: type, data: payload ?? type.value);
    final message = NUIMiniProgramMessage(channel: NUIMiniProgramMessage.DEFAULT_CHANNEL, data: jsonEncode(internalMessage));
    try {
      final result = await sendAndAwaitCrossEvent(message);
      return true;
    }catch(e){
      print("[NUIMiniProgramJSEngine] Failed to send ${type.value} event with error : $e");
      return false;
    }
  }

  Future<bool> sendPopEvent() async{
    final internalMessage = NUIMiniProgramInternalMessage(type: NUIMiniProgramEventType.POP, data: "pop");
    final popMessage = NUIMiniProgramMessage(channel: NUIMiniProgramMessage.DEFAULT_CHANNEL, data: jsonEncode(internalMessage));
    try {
      final result = await sendAndAwaitCrossEvent(popMessage);
      return true;
    }catch(e){
      print("[NUIMiniProgramJSEngine] Failed to send pop event with error : $e");
      return false;
    }
  }

  Future<NUIMiniProgramFile> sendFileSelect({List<String> extensions}) async{
    final internalMessage = NUIMiniProgramInternalMessage(type: NUIMiniProgramEventType.FILE_SELECT, data: "fileSelect");
    final popMessage = NUIMiniProgramMessage(channel: NUIMiniProgramMessage.DEFAULT_CHANNEL, data: jsonEncode(internalMessage));
    try {
      final result = await sendAndAwaitCrossEvent(popMessage);
      if(result.data != null){
        final json = result.data as Map;
        final filename = json["filename"] as String;
        final extensions = List<String>.from(json["extensions"] ?? []);
        final bytes = json["bytes"] as String;
        try {
          final fileMessage = NUIMiniProgramFile(filename: filename, bytesString: bytes, extensions: extensions);
          return fileMessage;
        }catch(e, s){
          print("[NUIMiniProgramJSEngine] Failed to parse result data payload to file object with error : $e, stacktrace: $s");
          return null;
        }
      }
      else{
        print("[NUIMiniProgramJSEngine] Gotten result without data payload");
        return null;
      }
    }catch(e, s){
      print("[NUIMiniProgramJSEngine] Failed to send file select event with error : $e, stacktrace: $s");
      return null;
    }
  }

  Future<bool> sendInitializedEvent() async{
    final internalMessage = NUIMiniProgramInternalMessage(type: NUIMiniProgramEventType.INITIALIZED, data: "initialized");
    final initMessage = NUIMiniProgramMessage(channel: NUIMiniProgramMessage.DEFAULT_CHANNEL, data: jsonEncode(internalMessage));
    try {
      final maxDelay = Future.delayed(Duration(milliseconds: 100), () async{
        return true;
      });

      final completer = Completer<bool>();
      sendAndAwaitCrossEvent(initMessage).then((value) {
        print("NUIMiniProgramWebJSEngine: Received initialized response from host");
        completer.complete(true);
      });

      await maxDelay;
      if(completer.isCompleted){
        final init = await completer.future;
        if(init){
          _connectionInitialized = true;
          return true;
        }
        return false;
      }
      return false;
    }catch(e){
      print("[NUIMiniProgramJSEngine] Failed to send initialized event with error : $e");
      return false;
    }
  }

  @override
  Future<bool> sendPlatformCrossEvent(NUIMiniProgramMessage message) async{
    try {
      final result = await js.context.callMethod('postNUIMessage', [jsonEncode(message)]);
      return true;
    }catch(e){
      print("[NUIMiniProgramJSEngine] Failed to post NUI Message with error : $e");
      return false;
    }
  }

}